@@ -290,49 +290,46 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 
 struct MeshBufList
 {
-	/*!
-	 * Specifies in which layer the list is.
-	 * All lists which are in a lower layer are rendered before this list.
-	 */
-	u8 layer;
 	video::SMaterial m;
 	std::vector<scene::IMeshBuffer*> bufs;
 };
 
 struct MeshBufListList
 {
-	std::vector<MeshBufList> lists;
+	/*!
+	 * Stores the mesh buffers of the world.
+	 * The array index is the material's layer.
+	 * The vector part groups vertices by material.
+	 */
+	std::vector<MeshBufList> lists[MAX_TILE_LAYERS];
 
 	void clear()
 	{
-		lists.clear();
+		for (int l = 0; l < MAX_TILE_LAYERS; l++)
+			lists[l].clear();
 	}
 
 	void add(scene::IMeshBuffer *buf, u8 layer)
 	{
+		// Append to the correct layer
+		std::vector<MeshBufList> &list = lists[layer];
 		const video::SMaterial &m = buf->getMaterial();
-		for(std::vector<MeshBufList>::iterator i = lists.begin();
-				i != lists.end(); ++i){
-			MeshBufList &l = *i;
-
+		for (std::vector<MeshBufList>::iterator it = list.begin(); it != list.end();
+			++it) {
 			// comparing a full material is quite expensive so we don't do it if
 			// not even first texture is equal
-			if (l.m.TextureLayer[0].Texture != m.TextureLayer[0].Texture)
+			if ((*it).m.TextureLayer[0].Texture != m.TextureLayer[0].Texture)
 				continue;
 
-			if(l.layer != layer)
-				continue;
-
-			if (l.m == m) {
-				l.bufs.push_back(buf);
+			if ((*it).m == m) {
+				(*it).bufs.push_back(buf);
 				return;
 			}
 		}
 		MeshBufList l;
-		l.layer = layer;
 		l.m = m;
 		l.bufs.push_back(buf);
-		lists.push_back(l);
+		list.push_back(l);
 	}
 };
 
@@ -360,7 +357,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 		Measuring time is very useful for long delays when the
 		machine is swapping a lot.
 	*/
-	int time1 = time(0);
+	time_t time1 = time(0);
 
 	/*
 		Get animation parameters
@@ -476,35 +473,34 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 		}
 	}
 
-	std::vector<MeshBufList> &lists = drawbufs.lists;
+	// Render all layers in order
+	for (int layer = 0; layer < MAX_TILE_LAYERS; layer++) {
+		std::vector<MeshBufList> &lists = drawbufs.lists[layer];
 
-	int timecheck_counter = 0;
-	for (std::vector<MeshBufList>::iterator i = lists.begin();
-			i != lists.end(); ++i) {
-		timecheck_counter++;
-		if (timecheck_counter > 50) {
-			timecheck_counter = 0;
-			int time2 = time(0);
-			if (time2 > time1 + 4) {
-				infostream << "ClientMap::renderMap(): "
-					"Rendering takes ages, returning."
-					<< std::endl;
-				return;
+		int timecheck_counter = 0;
+		for (std::vector<MeshBufList>::iterator it = lists.begin(); it != lists.end();
+			++it) {
+			timecheck_counter++;
+			if (timecheck_counter > 50) {
+				timecheck_counter = 0;
+				time_t time2 = time(0);
+				if (time2 > time1 + 4) {
+					infostream << "ClientMap::renderMap(): "
+						"Rendering takes ages, returning."
+						<< std::endl;
+					return;
+				}
+			}
+
+			driver->setMaterial((*it).m);
+
+			for (std::vector<scene::IMeshBuffer*>::iterator it2 = (*it).bufs.begin();
+				it2 != (*it).bufs.end(); ++it2) {
+				driver->drawMeshBuffer(*it2);
+				vertex_count += (*it2)->getVertexCount();
+				meshbuffer_count++;
 			}
 		}
-
-		MeshBufList &list = *i;
-
-		driver->setMaterial(list.m);
-
-		for (std::vector<scene::IMeshBuffer*>::iterator j = list.bufs.begin();
-				j != list.bufs.end(); ++j) {
-			scene::IMeshBuffer *buf = *j;
-			driver->drawMeshBuffer(buf);
-			vertex_count += buf->getVertexCount();
-			meshbuffer_count++;
-		}
-
 	}
 	} // ScopeProfiler
 
