@@ -59,7 +59,7 @@ public:
 		m_age += dtime;
 		if(m_age > 10)
 		{
-			m_removed = true;
+			m_pending_removal = true;
 			return;
 		}
 
@@ -406,20 +406,6 @@ void LuaEntitySAO::step(float dtime, bool send_recommended)
 		m_env->getScriptIface()->luaentity_Step(m_id, dtime);
 	}
 
-	// Remove LuaEntity beyond terrain edges
-	{
-		ServerMap *map = dynamic_cast<ServerMap *>(&m_env->getMap());
-		assert(map);
-		if (!m_pending_deactivation &&
-				map->saoPositionOverLimit(m_base_position)) {
-			infostream << "Remove SAO " << m_id << "(" << m_init_name
-				<< "), outside of limits" << std::endl;
-			m_pending_deactivation = true;
-			m_removed = true;
-			return;
-		}
-	}
-
 	if(send_recommended == false)
 		return;
 
@@ -555,9 +541,9 @@ int LuaEntitySAO::punch(v3f dir,
 		ServerActiveObject *puncher,
 		float time_from_last_punch)
 {
-	if (!m_registered){
+	if (!m_registered) {
 		// Delete unknown LuaEntities when punched
-		m_removed = true;
+		m_pending_removal = true;
 		return 0;
 	}
 
@@ -601,7 +587,7 @@ int LuaEntitySAO::punch(v3f dir,
 	}
 
 	if (getHP() == 0)
-		m_removed = true;
+		m_pending_removal = true;
 
 
 
@@ -1361,11 +1347,10 @@ void PlayerSAO::setWieldIndex(int i)
 	}
 }
 
-// Erase the peer id and make the object for removal
 void PlayerSAO::disconnected()
 {
 	m_peer_id = 0;
-	m_removed = true;
+	m_pending_removal = true;
 }
 
 void PlayerSAO::unlinkPlayerSessionAndSave()
@@ -1402,26 +1387,38 @@ bool PlayerSAO::checkMovementCheat()
 		too, and much more lightweight.
 	*/
 
-	float player_max_speed = 0;
+	float player_max_walk = 0; // horizontal movement
+	float player_max_jump = 0; // vertical upwards movement
 
-	if (m_privs.count("fast") != 0) {
-		// Fast speed
-		player_max_speed = m_player->movement_speed_fast * m_physics_override_speed;
-	} else {
-		// Normal speed
-		player_max_speed = m_player->movement_speed_walk * m_physics_override_speed;
-	}
-	// Tolerance. The lag pool does this a bit.
-	//player_max_speed *= 2.5;
+	if (m_privs.count("fast") != 0)
+		player_max_walk = m_player->movement_speed_fast; // Fast speed
+	else
+		player_max_walk = m_player->movement_speed_walk; // Normal speed
+	player_max_walk *= m_physics_override_speed;
+	player_max_jump = m_player->movement_speed_jump * m_physics_override_jump;
+	// FIXME: Bouncy nodes cause practically unbound increase in Y speed,
+	//        until this can be verified correctly, tolerate higher jumping speeds
+	player_max_jump *= 2.0;
+
+	// Don't divide by zero!
+	if (player_max_walk < 0.0001f)
+		player_max_walk = 0.0001f;
+	if (player_max_jump < 0.0001f)
+		player_max_jump = 0.0001f;
 
 	v3f diff = (m_base_position - m_last_good_position);
 	float d_vert = diff.Y;
 	diff.Y = 0;
 	float d_horiz = diff.getLength();
-	float required_time = d_horiz / player_max_speed;
+	float required_time = d_horiz / player_max_walk;
 
-	if (d_vert > 0 && d_vert / player_max_speed > required_time)
-		required_time = d_vert / player_max_speed; // Moving upwards
+	// FIXME: Checking downwards movement is not easily possible currently,
+	//        the server could calculate speed differences to examine the gravity
+	if (d_vert > 0) {
+		// In certain cases (water, ladders) walking speed is applied vertically
+		float s = MYMAX(player_max_jump, player_max_walk);
+		required_time = MYMAX(required_time, d_vert / s);
+	}
 
 	if (m_move_pool.grab(required_time)) {
 		m_last_good_position = m_base_position;
