@@ -13,11 +13,11 @@ void ItemStackMetadata::serialize(std::ostream &os) const
 {
 	std::ostringstream os2;
 	os2 << DESERIALIZE_START;
-	for (StringMap::const_iterator
-			it = m_stringvars.begin();
-			it != m_stringvars.end(); ++it) {
-		os2 << it->first << DESERIALIZE_KV_DELIM
-		    << it->second << DESERIALIZE_PAIR_DELIM;
+	for (StringMap::const_iterator it = m_stringvars.begin(); it != m_stringvars.end();
+		++it) {
+		if (!(*it).first.empty() || !(*it).second.empty())
+			os2 << (*it).first << DESERIALIZE_KV_DELIM
+				<< (*it).second << DESERIALIZE_PAIR_DELIM;
 	}
 	os << serializeJsonStringIfNeeded(os2.str());
 }
@@ -28,16 +28,18 @@ void ItemStackMetadata::deSerialize(std::istream &is)
 
 	m_stringvars.clear();
 
-	if (!in.empty() && in[0] == DESERIALIZE_START) {
-		Strfnd fnd(in);
-		fnd.to(1);
-		while (!fnd.at_end()) {
-			std::string name = fnd.next(DESERIALIZE_KV_DELIM_STR);
-			std::string var  = fnd.next(DESERIALIZE_PAIR_DELIM_STR);
-			m_stringvars[name] = var;
+	if (!in.empty()) {
+		if (in[0] == DESERIALIZE_START) {
+			Strfnd fnd(in);
+			fnd.to(1);
+			while (!fnd.at_end()) {
+				std::string name = fnd.next(DESERIALIZE_KV_DELIM_STR);
+				std::string var  = fnd.next(DESERIALIZE_PAIR_DELIM_STR);
+				m_stringvars[name] = var;
+			}
+		} else {
+			// BACKWARDS COMPATIBILITY
+			m_stringvars[""] = in;
 		}
-	} else {
-		// BACKWARDS COMPATIBILITY
-		m_stringvars[""] = in;
 	}
 }
