@@ -580,10 +580,6 @@ void Client::handleCommand_MovePlayer(NetworkPacket* pkt)
 	event.player_force_move.pitch = pitch;
 	event.player_force_move.yaw = yaw;
 	m_client_event_queue.push(event);
-
-	// Ignore damage for a few seconds, so that the player doesn't
-	// get damage from falling on ground
-	m_ignore_damage_timer = 3.0;
 }
 
 void Client::handleCommand_DeathScreen(NetworkPacket* pkt)
@@ -1172,9 +1168,21 @@ void Client::handleCommand_HudSetParam(NetworkPacket* pkt)
 			player->hud_hotbar_itemcount = hotbar_itemcount;
 	}
 	else if (param == HUD_PARAM_HOTBAR_IMAGE) {
+		// If value not empty verify image exists in texture source
+		if (value != "" && !getTextureSource()->isKnownSourceImage(value)) {
+			errorstream << "Server sent wrong Hud hotbar image (sent value: '"
+				<< value << "')" << std::endl;
+			return;
+		}
 		player->hotbar_image = value;
 	}
 	else if (param == HUD_PARAM_HOTBAR_SELECTED_IMAGE) {
+		// If value not empty verify image exists in texture source
+		if (value != "" && !getTextureSource()->isKnownSourceImage(value)) {
+			errorstream << "Server sent wrong Hud hotbar selected image (sent value: '"
+					<< value << "')" << std::endl;
+			return;
+		}
 		player->hotbar_selected_image = value;
 	}
 }
