@@ -16,13 +16,13 @@ toolchain_file=$dir/toolchain_mingw.cmake
 irrlicht_version=1.8.4
 ogg_version=1.3.2
 vorbis_version=1.3.5
-curl_version=7.50.3
+curl_version=7.54.0
 gettext_version=0.19.8.1
-freetype_version=2.7
-sqlite3_version=3.14.2
-luajit_version=2.1.0-beta2
-leveldb_version=1.18
-zlib_version=1.2.8
+freetype_version=2.8
+sqlite3_version=3.19.2
+luajit_version=2.1.0-beta3
+leveldb_version=1.19
+zlib_version=1.2.11
 
 mkdir -p $packagedir
 mkdir -p $libdir
@@ -149,6 +149,9 @@ cmake .. \
 	-DLEVELDB_LIBRARY=$libdir/leveldb/lib/libleveldb.dll.a \
 	-DLEVELDB_DLL=$libdir/leveldb/bin/libleveldb.dll
 
-make package -j2
+make -j2
 
+[ "x$NO_PACKAGE" = "x" ] && make package
+
+exit 0
 # EOF
